@@ -20,9 +20,9 @@
 """ Custom script for Thunderbird 3.
 """
 
-__id__        = "$Id: script.py 4221 2008-09-15 08:11:23Z wwalker $"
-__version__   = "$Revision: 4221 $"
-__date__      = "$Date: 2008-09-15 04:11:23 -0400 (Mon, 15 Sep 2008) $"
+__id__        = "$Id: script.py 4600 2009-02-25 16:13:21Z joanied $"
+__version__   = "$Revision: 4600 $"
+__date__      = "$Date: 2009-02-25 11:13:21 -0500 (Wed, 25 Feb 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -228,18 +228,29 @@ class Script(Gecko.Script):
 
         # This is a better fix for bug #405541. Thunderbird gives
         # focus to the cell in the column that is being sorted
-        # (e.g., Date). Braille should show the row from the begining.
-        # This fix calls orca.setLocusOfFocus to give focus to the
-        # cell at the beginning of the row. It consume the event
-        # so Gecko.py doesn't reset the focus.
+        # (e.g., Date). Braille should show the row beginning with
+        # the first populated cell. Set the locusOfFocus to that
+        # cell and consume the event so that the Gecko script
+        # doesn't reset it.
         #
         if obj.getRole() == pyatspi.ROLE_TABLE_CELL:
             table = parent.queryTable()
-            index = self.getCellIndex(obj)
-            row = table.getRowAtIndex(index)
-            acc = table.getAccessibleAt(row, 0)
-            orca.setLocusOfFocus(event, acc)
-            consume = True
+            row = table.getRowAtIndex(self.getCellIndex(obj))
+            for i in range(0, table.nColumns):
+                acc = table.getAccessibleAt(row, i)
+                if acc.name:
+                    # For some reason orca.py's check to see if the
+                    # object we're setting the locusOfFocus to is the
+                    # same as the current locusOfFocus is returning
+                    # True when it's not actually True. Therefore,
+                    # we'll set the current locusOfFocus to None as
+                    # a precaution.
+                    #
+                    if event.type.startswith("focus:"):
+                        orca_state.locusOfFocus = None
+                    orca.setLocusOfFocus(event, acc)
+                    consume = True
+                    break
 
         # Text area (for caching handle for spell checking purposes).
         #
