@@ -1,4 +1,4 @@
-# Copyright 2006, 2007, 2008 Brailcom, o.p.s.
+# Copyright 2006, 2007, 2008, 2009 Brailcom, o.p.s.
 #
 # Author: Tomas Cerha <cerha@brailcom.org>
 #
@@ -31,9 +31,9 @@
 NOTE: THIS IS EXPERIMENTAL ONLY AND IS NOT A SUPPORTED COMPONENT OF ORCA.
 """
 
-__id__ = "$Id: speechdispatcherfactory.py 4365 2008-11-24 15:30:25Z wwalker $"
-__version__   = "$Revision: 4365 $"
-__date__      = "$Date: 2008-11-24 10:30:25 -0500 (Mon, 24 Nov 2008) $"
+__id__ = "$Id: speechdispatcherfactory.py 4659 2009-03-30 17:37:48Z wwalker $"
+__version__   = "$Revision: 4659 $"
+__date__      = "$Date: 2009-03-30 13:37:48 -0400 (Mon, 30 Mar 2009) $"
 __author__    = "Tomas Cerha <cerha@brailcom.org>"
 __copyright__ = "Copyright (c) 2006-2008 Brailcom, o.p.s."
 __license__   = "LGPL"
@@ -90,10 +90,12 @@ class SpeechServer(speechserver.SpeechServer):
     getFactoryName = staticmethod(getFactoryName)
 
     def getSpeechServers():
+        servers = []
         default = SpeechServer._getSpeechServer(SpeechServer.DEFAULT_SERVER_ID)
-        servers = [default]
-        for module in default.list_output_modules():
-            servers.append(SpeechServer._getSpeechServer(module))
+        if default is not None:
+            servers.append(default)
+            for module in default.list_output_modules():
+                servers.append(SpeechServer._getSpeechServer(module))
         return servers
     getSpeechServers = staticmethod(getSpeechServers)
 
@@ -326,6 +328,7 @@ class SpeechServer(speechserver.SpeechServer):
     def speak(self, text=None, acss=None, interrupt=True):
         #if interrupt:
         #    self._cancel()
+
         if text:
             self._speak(text, acss)
 
