@@ -19,16 +19,15 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 4066 2008-07-26 23:35:05Z wwalker $"
-__version__   = "$Revision: 4066 $"
-__date__      = "$Date: 2008-07-26 19:35:05 -0400 (Sat, 26 Jul 2008) $"
+__id__        = "$Id: where_am_I.py 4674 2009-04-10 13:39:24Z wwalker $"
+__version__   = "$Revision: 4674 $"
+__date__      = "$Date: 2009-04-10 09:39:24 -0400 (Fri, 10 Apr 2009) $"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
 import debug
 import orca_state
-import rolenames
 import settings
 import speech
 import text_attribute_names
@@ -36,7 +35,7 @@ import urlparse, urllib2
 
 from orca_i18n import _ # for gettext support
 from orca_i18n import ngettext  # for ngettext support
-from orca_i18n import Q_ # to provide qualified translatable strings
+from orca_i18n import C_ # to provide qualified translatable strings
 
 class WhereAmI:
 
@@ -159,6 +158,8 @@ class WhereAmI:
         if basicOnly:
             self._speakObjDescription(obj)
 
+        self._lastAttributeString = ""
+
         return True
 
     def _speakCheckBox(self, obj, basicOnly):
@@ -168,11 +169,12 @@ class WhereAmI:
         2. role
         3. state
         4. accelerator (i.e. Alt plus the underlined letter), if any
+        5. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj) + " " + \
-               rolenames.getSpeechForRoleName(obj)
+               self._getSpeechForRoleName(obj)
         text = text + " " + self._getCheckBoxState(obj)
         utterances.append(text)
 
@@ -183,6 +185,9 @@ class WhereAmI:
         if text:
             utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "check box utterances=%s" \
                       % utterances)
         speech.speakUtterances(utterances)
@@ -197,6 +202,7 @@ class WhereAmI:
         4. state
         5. relative position
         6. accelerator (i.e. Alt plus the underlined letter), if any
+        7. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
@@ -208,7 +214,7 @@ class WhereAmI:
             utterances.append(text)
 
         text = self.getObjLabelAndName(obj) + " " + \
-               rolenames.getSpeechForRoleName(obj)
+               self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         state = obj.getState()
@@ -216,16 +222,12 @@ class WhereAmI:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            text = Q_("radiobutton|selected")
+            text = C_("radiobutton", "selected")
         else:
             # Translators: this is in reference to a radio button being
             # selected or not.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            text = Q_("radiobutton|not selected")
+            text = C_("radiobutton", "not selected")
 
         utterances.append(text)
 
@@ -235,6 +237,9 @@ class WhereAmI:
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "radio button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -248,13 +253,14 @@ class WhereAmI:
         3. current value
         4. relative position
         5. accelerator (i.e. Alt plus the underlined letter), if any
+        6. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         name = self._getObjName(obj)
@@ -267,6 +273,9 @@ class WhereAmI:
         accelerator = self._getObjAccelerator(obj)
         utterances.append(accelerator)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "combo box utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -280,13 +289,14 @@ class WhereAmI:
         3. current value
         4. selected (if True).
         5. accelerator (i.e. Alt plus the underlined letter), if any
+        6. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         label = self._getObjLabel(obj)
         utterances.append(label)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         name = self._getObjName(obj)
@@ -302,6 +312,9 @@ class WhereAmI:
         if text:
             utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "spin button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -313,18 +326,22 @@ class WhereAmI:
         1. label
         2. role
         3. accelerator (i.e. Alt plus the underlined letter), if any
+        4. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "push button utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -338,13 +355,14 @@ class WhereAmI:
         3. value
         4. percentage (if possible)
         5. accelerator (i.e. Alt plus the underlined letter), if any
+        6. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self._getObjLabel(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         values = self._getSliderValues(obj)
@@ -360,6 +378,9 @@ class WhereAmI:
         if text:
             utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "slider utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -374,11 +395,12 @@ class WhereAmI:
         accelerator key binding, if any
         3. relative position
         4. mnemonic (i.e. the underlined letter), if any
+        5. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj.parent) + " " + \
-               rolenames.getSpeechForRoleName(obj.parent)
+               self._getSpeechForRoleName(obj.parent)
         utterances.append(text.strip())
 
         text = self.getObjLabelAndName(obj)
@@ -387,7 +409,7 @@ class WhereAmI:
         state = obj.getState()
 
         if obj.getRole() != pyatspi.ROLE_MENU_ITEM:
-            text = rolenames.getSpeechForRoleName(obj)
+            text = self._getSpeechForRoleName(obj)
             utterances.append(text)
 
         if obj.getRole() == pyatspi.ROLE_CHECK_MENU_ITEM:
@@ -431,6 +453,9 @@ class WhereAmI:
             text = self._getObjMnemonic(obj)
             utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "menu item utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -443,10 +468,11 @@ class WhereAmI:
         2. label + 'page'
         3. relative position
         4. accelerator (i.e. Alt plus the underlined letter), if any
+        5. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
-        text = rolenames.getSpeechForRoleName(obj.parent)
+        text = self._getSpeechForRoleName(obj.parent)
         utterances.append(text)
 
         # Translators: "page" is the word for a page tab in a tab list.
@@ -461,6 +487,9 @@ class WhereAmI:
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "page utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -478,6 +507,7 @@ class WhereAmI:
             by 'selected' (single press)
             C. if the current line is blank/empty, 'blank'
         4. accelerator (i.e. Alt plus the underlined letter), if any
+        5. tutorial string if enableTutorialMessages is set.
 
         Gaim, gedit, OpenOffice Writer and Terminal
         """
@@ -504,7 +534,7 @@ class WhereAmI:
            and self._script.isReadOnlyTextArea(obj):
             utterances.append(settings.speechReadOnlyString)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         [textContents, startOffset, endOffset, selected] = \
@@ -521,6 +551,7 @@ class WhereAmI:
         utterances.append(text)
         debug.println(self._debugLevel, "first text utterances=%s" % \
                       utterances)
+
         speech.speakUtterances(utterances)
 
         if not basicOnly:
@@ -531,14 +562,15 @@ class WhereAmI:
             # Translators: when the user selects (highlights) text in
             # a document, Orca lets them know this.
             #
-            # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-            #
-            text = Q_("text|selected")
+            text = C_("text", "selected")
             utterances.append(text)
 
         text = self._getObjAccelerator(obj)
         utterances.append(text)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "text utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -557,6 +589,7 @@ class WhereAmI:
            performing a detailed whereAmI.
         8. if expandable/collapsible: expanded/collapsed
         9. if applicable, the level
+        10. tutorial string if enableTutorialMessages is set.
 
         Nautilus and Gaim
         """
@@ -566,7 +599,7 @@ class WhereAmI:
             obj = obj.parent
         parent = obj.parent
 
-        text = rolenames.getSpeechForRoleName(parent)
+        text = self._getSpeechForRoleName(parent)
         utterances.append(text)
 
         try:
@@ -589,7 +622,7 @@ class WhereAmI:
                 text = self._getObjName(header)
                 utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         if nColumns:
@@ -650,6 +683,9 @@ class WhereAmI:
             #
             utterances.append(_("tree level %d") % (level + 1))
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "third table cell utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -663,6 +699,7 @@ class WhereAmI:
         4. relative position
         5. if expandable/collapsible: expanded/collapsed
         6. if applicable, the level
+        7. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
@@ -671,7 +708,7 @@ class WhereAmI:
         if text:
             utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         text = self._getObjName(obj)
@@ -711,6 +748,9 @@ class WhereAmI:
             #
             utterances.append(_("tree level %d") % (level + 1))
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         debug.println(self._debugLevel, "list item utterances=%s" % \
                       utterances)
         speech.speakUtterances(utterances)
@@ -718,12 +758,15 @@ class WhereAmI:
     def _speakParagraph(self, obj, basicOnly):
         """Speak a paragraph object.
         """
+
         self._speakText(obj, basicOnly)
 
     def _speakIconPanel(self, obj, basicOnly):
         """Speak the contents of the pane containing this icon. The
-        number of icons in the pane is spoken. Plus the total number of
-        selected icons. Plus the name of each of the selected icons.
+        1. Number of icons in the pane is spoken.
+        2. The total number of selected icons.
+        3. The name of each of the selected icons.
+        4. tutorial string if enableTutorialMessages is set.
 
         Arguments:
         - obj: the icon object that currently has focus.
@@ -737,12 +780,20 @@ class WhereAmI:
         utterances.append(self.getObjLabelAndName(obj))
         utterances.extend(self._getSelectedItemCount(panel, basicOnly))
 
+        # get our tutorial.
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+        # get the frames tutorial.
+        [frame, dialogue] = self._getFrameAndDialog(obj)
+        utterances.extend(getTutorial(frame, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
         
     def _speakLink(self, obj, basicOnly):
         """Speaks information about a link including protocol, domain 
-        comparisons and size of file if possible
-        
+        comparisons and size of file if possible.
+        Also tutorial string if enableTutorialMessages is set.
+
         Arguments:
         - obj: the icon object that currently has focus.
         - basicOnly: True if the user is performing a standard/basic whereAmI.
@@ -771,6 +822,23 @@ class WhereAmI:
         # Translators: this is the protocol of a link eg. http, mailto.
         #
         linkoutput = _('%s link') %link_uri_info[0]
+        text = self._script.getDisplayedText(obj)
+        if text:
+            linkoutput += " " + text
+        else:
+            # If there's no text for the link, expose part of the
+            # URI to the user.
+            #
+            basename = self._script.getLinkBasename(obj)
+            if basename:
+                linkoutput += " " + basename
+
+        # If the link has a child which is an image, we want
+        # to indicate that.
+        #
+        if obj.childCount and obj[0].getRole() == pyatspi.ROLE_IMAGE:
+            linkoutput += " " + self._getSpeechForRoleName(obj[0])
+
         domainoutput = ''
         sizeoutput = ''
       
@@ -804,20 +872,25 @@ class WhereAmI:
                 else:
                     domainoutput = _('different site')
 
-        speech.speakUtterances([linkoutput, domainoutput, sizeoutput])
+        utterances = [linkoutput, domainoutput, sizeoutput]
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
+        speech.speakUtterances(utterances)
 
     def _speakToggleButton(self, obj, basicOnly):
         """Speak toggle button information:
            1. Name/Label
            2. Role
            3. State (pressed/not pressed)
+           4. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         if obj.getState().contains(pyatspi.STATE_CHECKED):
@@ -830,6 +903,9 @@ class WhereAmI:
             checkedState = _("not pressed")
         utterances.append(checkedState)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
       
     def _speakSplitPane(self, obj, basicOnly):
@@ -837,18 +913,22 @@ class WhereAmI:
            1. Name/Label
            2. Role
            3. Value
+           4. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
 
         valueString = self._script.getTextForValue(obj)
         utterances.append(valueString)
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
 
     def _speakLabel(self, obj, basicOnly):
@@ -856,6 +936,7 @@ class WhereAmI:
            1. Name/Label
            2. selected (if True).
            3. Role
+           4. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
@@ -864,27 +945,37 @@ class WhereAmI:
 
         utterances.extend(self._getSpeechForAllTextSelection(obj))
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
+
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
 
+
     def _speakLayeredPane(self, obj, basicOnly):
         """Speak layered pane information:
            1. Name/Label
            2. Role
            3. Number of selected items and total number of items.
+           4. tutorial string if enableTutorialMessages is set.
         """
 
         utterances = []
         text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
         utterances.extend(self._getSelectedItemCount(obj, basicOnly))
 
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
 
+
     def _getSpeechForAllTextSelection(self, obj):
         """Check if this object has text associated with it and it's
         completely selected.
@@ -907,9 +998,7 @@ class WhereAmI:
                     # Translators: when the user selects (highlights) text in
                     # a document, Orca lets them know this.
                     #
-                    # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
-                    #
-                    utterance = [Q_("text|selected")]
+                    utterance = [C_("text", "selected")]
 
         return utterance
 
@@ -1002,10 +1091,15 @@ class WhereAmI:
         text = self.getObjLabelAndName(obj)
         utterances.append(text)
 
-        text = rolenames.getSpeechForRoleName(obj)
+        text = self._getSpeechForRoleName(obj)
         utterances.append(text)
+
+        getTutorial = self._script.tutorialGenerator.getTutorial
+        utterances.extend(getTutorial(obj, False, forceMessage=True))
+
         speech.speakUtterances(utterances)
 
+
     def _getObjName(self, obj):
         """Returns the name to speak for an object.
         """
@@ -1060,6 +1154,16 @@ class WhereAmI:
 
         return text.strip()
 
+    def _getSpeechForRoleName(self, obj, role=None):
+        """Returns the rolename to be spoken for the object.
+        """
+
+        try:
+            return self._script.speechGenerator.\
+                getSpeechForObjectRole(obj, role)[0]
+        except:
+            return ""
+
     def _getGroupLabel(self, obj):
         """Returns the label for a group of components.
         """
@@ -1208,6 +1312,7 @@ class WhereAmI:
     def _getTableRow(self, obj):
         """Get the speech for a table cell row or a single table cell
         if settings.readTableCellRow is False.
+        Also return a tutorial string if enableTutorialMessages is set.
 
         Arguments:
         - obj: the table
@@ -1233,6 +1338,8 @@ class WhereAmI:
 
             debug.println(self._debugLevel, "row=<%s>" % utterances)
 
+            getTutorial = self._script.tutorialGenerator.getTutorial
+            utterances.extend(getTutorial(obj, False, forceMessage=True))
             return utterances
 
     def _getTableCell(self, obj):
@@ -1255,14 +1362,13 @@ class WhereAmI:
                     break
 
         if isToggle:
-            text = rolenames.getSpeechForRoleName(obj, pyatspi.ROLE_CHECK_BOX)
+            text = self._getSpeechForRoleName(obj, pyatspi.ROLE_CHECK_BOX)
             text = text + " " + self._getCheckBoxState(obj)
         else:
             descendant = self._script.getRealActiveDescendant(obj)
             text = self._script.getDisplayedText(descendant)
 
         debug.println(self._debugLevel, "cell=<%s>" % text)
-
         return text
 
     def _getCheckBoxState(self, obj):
@@ -1548,49 +1654,69 @@ class WhereAmI:
             debug.println(self._debugLevel,
                           "line attribs <%s>" % (attribs))
             if attribs:
-                newLine += " ; "
+                if newLine:
+                    newLine += " ; "
                 newLine += attribs
                 newLine += " "
 
             newLine += line[i]
             textOffset += 1
 
+        attribs = self._getAttributesForChar(obj,
+                                             text,
+                                             startOffset,
+                                             line,
+                                             0,
+                                             ["paragraph-style"])
+        if attribs:
+            if newLine:
+                newLine += " ; "
+            newLine += attribs
+
         debug.println(self._debugLevel, "newLine: <%s>" % (newLine))
 
         return newLine
 
-    def _getAttributesForChar(self, obj, text, textOffset, line, lineIndex):
+    def _getAttributesForChar(self,
+                              obj,
+                              text,
+                              textOffset,
+                              line,
+                              lineIndex,
+                              keys=["style", "weight", "underline"]):
 
-        keys = [ "style", "weight", "underline" ]
         attribStr = ""
-        charAttributes = text.getAttributes(textOffset)
 
+        defaultAttributes = text.getDefaultAttributes()
+        attributesDictionary = self._stringToDictionary(defaultAttributes)
+
+        charAttributes = text.getAttributes(textOffset)
         if charAttributes[0]:
             charDict = self._stringToDictionary(charAttributes[0])
-            debug.println(self._debugLevel,
-                          "charDict: %s" % (charDict))
+            for key in charDict.keys():
+                attributesDictionary[key] = charDict[key]
 
+        if attributesDictionary:
             for key in keys:
                 localizedKey = text_attribute_names.getTextAttributeName(key)
-                if key in charDict:
-                    attribute = charDict[key]
+                if key in attributesDictionary:
+                    attribute = attributesDictionary[key]
                     localizedValue = \
                         text_attribute_names.getTextAttributeName(attribute)
                     if attribute:
                         # If it's the 'weight' attribute and greater than 400,
                         # just speak it as bold, otherwise speak the weight.
                         #
-                        if key == "weight" and int(attribute) > 400:
-                            attribStr += " "
-                            # Translators: bold as in the font sense.
-                            #
-                            attribStr += _("bold")
-
+                        if key == "weight":
+                            if int(attribute) > 400:
+                                attribStr += " "
+                                # Translators: bold as in the font sense.
+                                #
+                                attribStr += _("bold")
                         elif key == "underline":
                             if attribute != "none":
                                 attribStr += " "
                                 attribStr += localizedKey
-
                         elif key == "style":
                             if attribute != "normal":
                                 attribStr += " "
@@ -1608,8 +1734,9 @@ class WhereAmI:
                 #
                 attribStr += _("link")
 
-            debug.println(self._debugLevel,
-                          "char <%s>: %s" % (line[lineIndex], attribStr))
+            if line:
+                debug.println(self._debugLevel,
+                              "char <%s>: %s" % (line[lineIndex], attribStr))
 
         # Only return attributes for the beginning of an attribute run.
         if attribStr != self._lastAttributeString:
@@ -1846,7 +1973,7 @@ class WhereAmI:
 
         utterances = []
         text = self.getObjLabelAndName(obj) + " " + \
-               rolenames.getSpeechForRoleName(obj)
+               self._getSpeechForRoleName(obj)
         utterances.append(text.strip())
 
         debug.println(self._debugLevel, "toolbar utterances=%s" \
